@@ -1,4 +1,4 @@
-/*	$OpenBSD: bytestring.h,v 1.15 2016/11/04 18:28:58 guenther Exp $	*/
+/*	$OpenBSD: bytestring.h,v 1.15.6.1 2017/12/09 13:43:25 jsing Exp $	*/
 /*
  * Copyright (c) 2014, Google Inc.
  *
@@ -393,6 +393,12 @@ int CBB_finish(CBB *cbb, uint8_t **out_data, size_t *out_len);
  */
 int CBB_flush(CBB *cbb);
 
+/*
+ * CBB_discard_child discards the current unflushed child of |cbb|. Neither the
+ * child's contents nor the length prefix will be included in the output.
+ */
+void CBB_discard_child(CBB *cbb);
+
 /*
  * CBB_add_u8_length_prefixed sets |*out_contents| to a new child of |cbb|. The
  * data written to |*out_contents| will be prefixed in |cbb| with an 8-bit
