@@ -1,4 +1,4 @@
-/* $OpenBSD: d1_lib.c,v 1.41 2017/02/07 02:08:38 beck Exp $ */
+/* $OpenBSD: d1_lib.c,v 1.42 2017/04/10 17:27:33 jsing Exp $ */
 /*
  * DTLS implementation written by Nagendra Modadugu
  * (nagendra@cs.stanford.edu) for the OpenSSL project 2005.
@@ -191,11 +191,8 @@ dtls1_free(SSL *s)
 	pqueue_free(s->d1->sent_messages);
 	pqueue_free(D1I(s)->buffered_app_data.q);
 
-	explicit_bzero(s->d1->internal, sizeof(*s->d1->internal));
-	free(s->d1->internal);
-
-	explicit_bzero(s->d1, sizeof(*s->d1));
-	free(s->d1);
+	freezero(s->d1->internal, sizeof(*s->d1->internal));
+	freezero(s->d1, sizeof(*s->d1));
 
 	s->d1 = NULL;
 }
