@@ -1,4 +1,4 @@
-/* $OpenBSD: tls_config.c,v 1.44 2017/09/25 18:07:03 jsing Exp $ */
+/* $OpenBSD: tls_config.c,v 1.44.4.1 2017/12/09 16:49:17 jsing Exp $ */
 /*
  * Copyright (c) 2014 Joel Sing <jsing@openbsd.org>
  *
@@ -311,8 +311,10 @@ tls_config_parse_protocols(uint32_t *protocols, const char *protostr)
 	char *s, *p, *q;
 	int negate;
 
-	if (protostr == NULL)
-		return TLS_PROTOCOLS_DEFAULT;
+	if (protostr == NULL) {
+		*protocols = TLS_PROTOCOLS_DEFAULT;
+		return (0);
+	}
 
 	if ((s = strdup(protostr)) == NULL)
 		return (-1);
